const {JSONPath} = require('jsonpath-plus');
const Output = require('./Output.js');

class Parser {

  constructor(json) {
    this.json = "";
    if(typeof json === "string") {
      this.json = JSON.parse(json);
    } else {
      this.json = json;
    }
    this.output = new Output();
  }

  parse() {
    // get paths
    var paths = JSONPath({path: "$.paths.*~", json: this.json});

    paths.forEach(path => {
      this.parsePath(path); //eg: /auth/login
    });

    return this.output.getAbapCode();
  }


  parsePath(path) {
    // get operations
    var opertions = JSONPath({path: "$.paths." + path + ".*~", json: this.json});
    
    opertions.forEach(operation => {
      this.parseOperation(path, operation); //eg: POST
      this.output.finishPath();
    })
  }


  parseOperation(path, operation) {
    var summary = JSONPath({path: "$.paths." + path + "." + operation + ".summary", json: this.json});
    var description = JSONPath({path: "$.paths." + path + "." + operation + ".description", json: this.json});
    
    // operationID leer -> ERROR
    var operationId = JSONPath({path: "$.paths." + path + "." + operation + ".operationId", json: this.json});
    if(!operationId || operationId.length < 1) {
      throw 'ERROR: operationId of path: "' + path + '" missing';
    }

    // x-weco-structure leer -> ERROR
    var xWecoStructure = JSONPath({path: "$.paths." + path + "." + operation + ".x-weco-structure", json: this.json});
    if(!xWecoStructure || xWecoStructure.length < 1) {
      throw 'ERROR: x-weco-structure of path: "' + path + '" missing';
    }

    var security = JSONPath({path: "$.paths." + path + "." + operation + ".security.~", json: this.json});
    security = security[2];

    this.output.addPath(path, operation, operationId, xWecoStructure, security);

    // get parameters
    var parameters = JSONPath({path: "$.paths." + path + "." + operation + ".parameters.*", json: this.json});
    
    parameters.forEach(parameter => {
      if(parameter.$ref) {
        parameter = this.resolveParameterRef(parameter.$ref);
      }

      this.parseParameter(parameter);  
    });

    // get request body
    var requestBody = JSONPath({path: "$.paths." + path + "." + operation + ".requestBody", json: this.json});
    if(requestBody.length == 1) {
      this.parseRequestBody(path, operation, requestBody[0]);
    }
  }

  parseRequestBody(path, operation, requestBody) {
    // Request Body
    var required = requestBody.required;        
    var type = JSONPath({path: "$.paths." + path + "." + operation + ".requestBody.content.*~", json: this.json});
    this.output.addRequestBody(required, type[0]);

    // Request Body Properties
    var requiredProperties = JSONPath({path: "$.paths." + path + "." + operation + ".requestBody.content." + type + ".schema.required.*", json: this.json});

    var propertyNames = JSONPath({path: "$.paths." + path + "." + operation + ".requestBody.content." + type + ".schema.properties.*~", json: this.json});

    propertyNames.forEach(propertyName => {
      var propertyType = JSONPath({path: "$.paths." + path + "." + operation + ".requestBody.content."
                          + type + ".schema.properties." + propertyName + ".type", json: this.json});
      propertyType = propertyType[0];
      
      var propertyRequired = requiredProperties.includes(propertyName);
      this.output.addRequestBodyProperty(propertyName, propertyType, propertyRequired);
    });

    this.output.finishRequestBody();
  }


  parseParameter(parameter) {
    var name = parameter.name;
    var inSegment = parameter.in;
    var required = parameter.required ? 'X' : '';

    if(parameter.schema && parameter.schema.type) {
      var type = parameter.schema.type;
    }

    this.output.addParameter(name, inSegment, required, type);
  }

  resolveParameterRef(ref) {
    var path = ref;
    path = this.replaceAll(path, '#/', '');
    path = this.replaceAll(path, '/','.');

    var result = JSONPath({path: "$." + path, json: this.json});
    return result[0];
  }

  replaceAll(string, search, replacement) {
    return string.replace(new RegExp(search, 'g'), replacement);
  } 

}

module.exports = Parser;